import logging
import os

formatter = logging.Formatter(
    fmt="%(asctime)s - %(levelname)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
)
handler = logging.StreamHandler()
handler.setFormatter(formatter)
logger = logging.getLogger("qwazzock")
logger.setLevel(os.getenv("QWAZZOCK_LOG_LEVEL", "INFO").upper())
logger.addHandler(handler)
logger.debug("Logging configured")



from flask import Flask
from flask import render_template
from flask import request
from flask import send_file
from flask_socketio import emit
from flask_socketio import SocketIO

from qwazzock import logger


def get_socketio_and_app(game):
    app = Flask(__name__)
    socketio = SocketIO(app, cors_allowed_origins="*")

    # Routes

    @app.route("/")
    def player_client():
        return render_template("player_client.html", page_name="player_client")

    @app.route("/host")
    def host():
        return render_template("host_client.html", page_name="host_client")

    @app.route("/static/questions/<string:requested_image>")
    def send_question_image(requested_image):
        logger.info(f"Sending question image {requested_image} from content directory.")
        return send_file(f"{game.content_path}/questions/{requested_image}")

    # Sockets

    @socketio.on("connect", namespace="/host_client_socket")
    def host_client_socket_connect():
        update_clients()

    @socketio.on("pass", namespace="/host_client_socket")
    def host_client_socket_pass_event():
        game.clear_hotseat()
        game.locked_out_teams = []
        game.next_question()
        update_clients()

    @socketio.on("picture", namespace="/host_client_socket")
    def host_client_socket_picture_event():
        if game.selected_image_index is None:
            logger.error(
                "No question images available. Cannot enable picture question type."
            )
        else:
            logger.info("Setting question type to picture.")
            game.question_type = "picture"
        update_clients()

    @socketio.on("reset", namespace="/host_client_socket")
    def host_client_socket_reset_event():
        game.reset()
        update_clients()

    @socketio.on("right", namespace="/host_client_socket")
    def host_client_socket_right_event(json):
        score_value = int(json["score_value"])
        logger.info(f"Received right with score_value of {score_value}.")
        game.next_question()
        if score_value > 0 and score_value < 10:
            game.right_answer(score_value)
            update_clients()

    @socketio.on("standard", namespace="/host_client_socket")
    def host_client_socket_standard_event():
        logger.info("Setting question type to standard.")
        game.question_type = "standard"
        update_clients()

    @socketio.on("wrong", namespace="/host_client_socket")
    def host_client_socket_wrong_event():
        game.wrong_answer()
        update_clients()

    @socketio.on("connect", namespace="/player_client_socket")
    def player_client_socket_connect():
        update_clients()

    @socketio.on("buzz", namespace="/player_client_socket")
    def player_client_socket_buzz_event(json):
        logger.info(f"Received buzz from {json['player_name']} of {json['team_name']}.")
        game.update_hotseat(
            player_name=json["player_name"], team_name=json["team_name"]
        )
        update_clients()

    # Helpers

    def update_clients():
        socketio.emit(
            "host_client_data", game.__dict__, namespace="/host_client_socket"
        )
        socketio.emit(
            "player_client_data", game.__dict__, namespace="/player_client_socket"
        )

    return socketio, app



import operator
import os
import random

from qwazzock import logger


class Game:
    def __init__(self, content_path=None):
        self.content_path = content_path
        self.reset()

    def update_hotseat(self, player_name, team_name):
        player_name = player_name.strip()
        team_name = team_name.strip()
        if (
            self.player_in_hotseat == "Pending"
            and team_name not in self.locked_out_teams
        ):
            self.player_in_hotseat = player_name
            self.team_in_hotseat = team_name

    def clear_hotseat(self):
        self.player_in_hotseat = "Pending"
        self.team_in_hotseat = "Pending"

    def next_question(self):
        if self.question_type == "picture":
            self.selected_image_index += 1
            try:
                self.selected_image = self.question_images[self.selected_image_index]
            except IndexError:
                logger.info(
                    "No more picture questions available. Reverting to standard question type."
                )
                self.question_type = "standard"
                self.prepare_picture_round()

    def prepare_picture_round(self):
        self.selected_image_index = None
        try:
            self.question_images = os.listdir(f"{self.content_path}/questions")
        except FileNotFoundError:
            logger.warning(
                f"Could not find questions folder in content directory ({self.content_path}). Picture question type will not be available."
            )
            return
        if self.question_images:
            random.shuffle(self.question_images)
            self.selected_image = self.question_images[0]
        else:
            logger.warning(
                f"No question images available in content directory ({self.content_path}/questions). Picture question type will not be available."
            )
            return
        self.selected_image_index = 0

    def reset(self):
        self.clear_hotseat()
        self.scores = {}
        self.locked_out_teams = []
        self.question_type = "standard"
        self.prepare_picture_round()

    def right_answer(self, score_value=1):
        self.scores[self.team_in_hotseat] = (
            self.scores.get(self.team_in_hotseat, 0) + score_value
        )
        self.scores = dict(
            sorted(self.scores.items(), key=operator.itemgetter(1), reverse=True)
        )
        self.locked_out_teams = []
        self.clear_hotseat()

    def wrong_answer(self):
        self.locked_out_teams.append(self.team_in_hotseat)
        self.clear_hotseat()



import os

from qwazzock.logs import logger  # noreorder
from qwazzock.game import Game
from qwazzock.server import get_socketio_and_app


def run():
    game = Game(content_path=os.getenv("QWAZZOCK_CONTENT_PATH"))
    logger.info(f"Content path set to {game.content_path}.")
    socketio, app = get_socketio_and_app(game=game)
    debug_mode = True if os.getenv("QWAZZOCK_SOCKETIO_DEBUG_MODE") else False
    socketio.run(app, debug=debug_mode, host="0.0.0.0")


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qwazzock']

package_data = \
{'': ['*'],
 'qwazzock': ['static/audio/*',
              'static/css/*',
              'static/dialog-polyfill/*',
              'static/icons/*',
              'static/images/*',
              'static/js/*',
              'templates/*']}

install_requires = \
['eventlet>=0.25.2,<0.26.0',
 'flask-socketio>=4.2.1,<5.0.0',
 'flask>=1.1.2,<2.0.0',
 'pyopenssl>=19.1.0,<20.0.0']

entry_points = \
{'console_scripts': ['qwazzock = qwazzock:run']}

setup_kwargs = {
    'name': 'qwazzock',
    'version': '0.11.1',
    'description': 'Qwazzock quiz app.',
    'long_description': None,
    'author': 'Dave Randall',
    'author_email': '19395688+daveygit2050@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
